from chitra.trainer import Dataset

dataset = Dataset("./")


def test__process():
    assert True


def test__reload():
    assert True


def test__capture_return_types():
    assert True


def test_update_component():
    assert True


def test_get_labels():
    assert True


def test_label_encoder():
    assert True


def test_generator():
    assert True


def test_get_tf_dataset():
    assert True
