#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['chitra',
 'chitra.cli',
 'chitra.converter',
 'chitra.data_processing',
 'chitra.data_processing.default',
 'chitra.serve',
 'chitra.serve.cloud',
 'chitra.serve.schema',
 'chitra.serve.tf_serving',
 'chitra.serve.torchserve',
 'chitra.utility',
 'chitra.visualization']

package_data = \
{'': ['*'], 'chitra': ['assets/*']}

install_requires = \
['tensorflow >= 2.3',
 'tensorflow-addons >=0.13.0',
 'tf-keras-vis >=0.5.3',
 'pytorch-lightning',
 'matplotlib',
 'pillow',
 'imgaug >=0.4.0',
 'scikit-learn',
 'typeguard',
 'gradio >=2.2.2',
 'typer']

extras_require = \
{'converter': ['onnx', 'tf2onnx'],
 'serve': ['fastapi',
           'uvicorn',
           'pydantic',
           'python-multipart',
           'tensorflow-serving-api',
           'chalice',
           'smart_open'],
 'test': ['pytest', 'pytest-asyncio', 'coverage']}

entry_points = \
{'console_scripts': ['chitra = chitra.cli.main:app']}

setup(name='chitra',
      version='0.2.0a1',
      description='Deep Learning library for Model Building, Interpretability, Visualization, API Building & Deployment.',
      author='Aniket Maurya',
      author_email='hello@aniketmaurya.com',
      url='https://github.com/aniketmaurya/chitra',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
